/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.novabpm.identity;

import java.security.Permission;
import java.util.Properties;
import java.util.Set;

/**
 * @author "Pierre Vigneras"
 * @date Nov 23, 2007
 */
public abstract class GroupTest extends AbstractTest {

  /**
   * Test method for
   * {@link org.ow2.novabpm.identity.impl.GroupImpl#getChildren()}.
   */
  public void testRelationShip() {
    final GroupOp parent = newGroup();
    assertTrue(parent.getChildren().isEmpty());
    assertEquals(parent.getParent(), parent.getRoot());
    final GroupOp child = newGroup(parent);
    assertTrue(parent.getChildren().contains(child));
    assertEquals(child.getParent(), parent);
    assertNotSame(child.getParent(), child.getRoot());
    assertNotSame(child.getParent(), parent.getRoot());
    assertSame(child.getRoot(), parent.getRoot());
  }

  protected abstract GroupOp newGroup();

  protected abstract GroupOp newGroup(GroupOp parent);

  /**
   * Test method for
   * {@link org.ow2.novabpm.identity.impl.GroupImpl#getMemberships()}.
   */
  public void testGetMemberships() {
    final GroupOp groupOp = newGroup();
    final Set<Membership> memberships = groupOp.getMemberships();
    assertNotNull(memberships);
    testUnmodifiableSet(memberships, null);
  }

  /**
   * Test method for {@link org.ow2.novabpm.identity.impl.GroupImpl#getId()}.
   */
  public void testGetId() {
    final GroupOp groupOp = newGroup();
    assertNotNull(groupOp.getId());
  }

  /**
   * Test method for
   * {@link org.ow2.novabpm.identity.impl.GroupImpl#getPermissions()}.
   */
  public void testGetPermissions() {
    final GroupOp groupOp = newGroup();
    final Set<Permission> permissions = groupOp.getPermissions();
    assertNotNull(permissions);
    final Permission permission = new DummyPermission();
    permissions.add(permission);
    assertFalse("Returned permissions should be a copy!", groupOp
        .getPermissions().contains(permission));
    groupOp.setPermissions(permissions);
    assertTrue("Can't find the added permission!", groupOp.getPermissions()
        .contains(permission));
  }

  /**
   * Test method for
   * {@link org.ow2.novabpm.identity.impl.GroupImpl#getPreferences()}.
   */
  // public void testGetPreferences() {
  // final GroupOp group = newGroup();
  // assertNotNull(group.getPreferences());
  // }
  /**
   * Test method for
   * {@link org.ow2.novabpm.identity.impl.GroupImpl#getProperties()}.
   */
  public void testGetProperties() {
    final GroupOp groupOp = newGroup();
    final Properties properties = groupOp.getProperties();
    assertNotNull(properties);
    final String KEY = "properties";
    properties.put(KEY, "A");
    // Check that the returned properties is a copy of the actual one.
    assertEquals("getProperties() should return a copy!", "B", groupOp
        .getProperties().getProperty(KEY, "B"));
    groupOp.setProperties(properties);
  }
}
