/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.novabpm.identity;

import java.io.Serializable;
import java.security.Permission;
import java.util.Properties;
import java.util.Set;

/**
 * Represent a UserOp in NovaBonita.
 * 
 * @author "Pierre Vigneras"
 * @date Nov 23, 2007
 */
public interface UserOp extends Serializable {

  String getId();

  // Implement The Preference API but there is no need to specify it
  // in the UserOp API, it it somehow automatic
  //	
  // Preferences getPreferences();
  // void setPreferences(Preferences newPreferences);

  /**
   * Get the set of properties of this user.
   * 
   * The returned Properties is a copy of the actual one. Any modifications made
   * to the returned Properties are not propagated to the backing store.
   * 
   * @return the properties of this user
   */
  Properties getProperties();

  void setProperties(Properties newProperties);

  /**
   * Get the set of permissions of this user.
   * 
   * The return Permissions is a copy of the actual one. Any modifications made
   * to the returned Permissions are not propagated to the backing store.
   * 
   * @return the set of permissions of this user
   */
  Set<Permission> getPermissions();

  void setPermissions(Set<Permission> permissions);

  /**
   * Return the memberships of this user.
   * 
   * The return set is a copy of the actual one. Any modifications made to the
   * returned set are not propagated to the backing store.
   * 
   * @return a set of Membership
   * 
   * @see Membership
   */
  Set<Membership> getMemberships();

  /**
   * @param groupOp
   * @return
   */
  boolean isMember(GroupOp groupOp);

}
